use std::time::{Duration, Instant};

use bracket_lib::prelude::*;
use hecs::{Entity, With, Without, World};

use crate::{components::monster::*, components::*, constants::*, resources::*, systems};

pub fn try_move(delta_x: i32, delta_y: i32, world: &mut World, resources: &mut Resources) {
    let mut to_kill: Vec<Entity> = Vec::new();

    for (player_entity, (player, pos)) in &mut world.query::<(&mut Player, &mut Position)>() {
        // The player shouldn't be able to move while whipping
        if let Ok(_wants_to_whip) = world.get::<WantsToWhip>(player_entity) {
            continue;
        }

        let now = Instant::now();
        if now - player.last_moved > Duration::from_secs_f32(PLAYER_STEP_PERIOD) {
            let destination = Point {
                x: pos.x + delta_x,
                y: pos.y + delta_y,
            };

            if resources.map.in_bounds(destination) {
                if resources.map.is_solid(destination) {
                    resources
                        .sound_output
                        .play_sound(resources.sound_effects.blocked.clone());
                } else {
                    if let Some(monster_entity) = resources.map.get_tile_content_at(destination) {
                        if let Ok(monster) = world.get::<Monster>(monster_entity) {
                            resources.stats.add_score(damage_for_kind(monster.kind));
                            resources.stats.take_gems(damage_for_kind(monster.kind));
                            resources.sound_output.play_sound(sound_effect_for_kind(
                                monster.kind,
                                &resources.sound_effects,
                            ));
                            to_kill.push(monster_entity);
                        }
                    }

                    resources.map.clear_tile_content_at(Point::from(*pos));

                    pos.x = destination.x;
                    pos.y = destination.y;

                    resources
                        .map
                        .set_tile_content_at(destination, player_entity);

                    systems::time::force_tick(&mut resources.clock);

                    resources
                        .sound_output
                        .play_sound(resources.sound_effects.step.clone());
                }
            } else {
                let static_sound = resources
                    .sound_effects
                    .get_new_static_effect(&resources.sound_output);
                resources.sound_output.play_sound(static_sound);
            }

            player.last_moved = now;
        }
    }

    for e in to_kill {
        let _ = world.despawn(e);
    }
}

pub fn whip(world: &mut World, resources: &mut Resources) {
    let mut to_add: Vec<Entity> = Vec::new();

    for (entity, _) in world.query_mut::<With<Player, With<Position, Without<WantsToWhip, ()>>>>() {
        if resources.stats.whips > 0 {
            to_add.push(entity);
        }
    }

    for e in to_add {
        let _ = world.insert_one(
            e,
            WantsToWhip {
                frame: 0,
                last_frame: Instant::now(),
                sound: Some(
                    resources
                        .sound_output
                        .play_sound(resources.sound_effects.whipping.clone()),
                ),
            },
        );
        resources.stats.whips -= 1;
    }
}
